import {
  newTracker,
  enableActivityTracking,
  trackSelfDescribingEvent,
  trackPageView,
  setUserId,
  addGlobalContexts,
  disableAnonymousTracking,
  BrowserTracker,
  PageViewEvent,
  CommonEventProperties,
} from '@snowplow/browser-tracker';
import { enableFormTracking } from '@snowplow/browser-plugin-form-tracking';
import { enableLinkClickTracking } from '@snowplow/browser-plugin-link-click-tracking';
import {
  enableErrorTracking,
  trackError as _trackError,
} from '@snowplow/browser-plugin-error-tracking';

import { isEmpty } from './utils/isEmpty';
import { doNotTrackEnabled } from './utils/doNotTrack';
import { hasGlobalPrivacyControlConsent } from './utils/globalPrivacyControl';
import { SCHEMAS, DEFAULT_TRACKER_ID } from './utils/constants';
import { getSnowplowOptions } from './utils/getSnowplowOptions';
import {
  AllowedPlugins,
  GitLabClientSDKOptions,
  ErrorEventProperties,
  GitLabClientSDKType,
} from './types';

export function glClientSDK(
  options: GitLabClientSDKOptions
): GitLabClientSDKType {
  return new GitLabClientSDK(options);
}

class GitLabClientSDK {
  snowplow: BrowserTracker | null = null;
  options: GitLabClientSDKOptions = {
    appId: '',
    host: '',
    hasCookieConsent: false,
    respectGlobalPrivacyControl: true,
    trackerId: DEFAULT_TRACKER_ID,
    pagePingOptions: {
      minimumVisitLength: 30,
      heartbeatDelay: 10,
    },
    pagePingTracking: true,
  };
  defaultPlugins: AllowedPlugins = {
    formTracking: true,
    clientHints: true,
    linkTracking: true,
    performanceTiming: true,
    errorTracking: true,
  };

  constructor(options: GitLabClientSDKOptions) {
    this.options = { ...this.options, ...options };
    this.options.plugins = {
      ...this.defaultPlugins,
      ...(options?.plugins ?? {}),
    };

    if (!this.trackingEnabled) return;

    const {
      appId,
      host,
      hasCookieConsent = false,
      trackerId = DEFAULT_TRACKER_ID,
      pagePingTracking,
      pagePingOptions,
      plugins,
    } = this.options;

    if (!appId) {
      console.warn('GitLab: No appId was provided');
      return;
    }

    if (!host) {
      console.warn('GitLab: No host was provided');
      return;
    }

    /**
     * Initialize Snowplow Client with options
     */
    const snowplowOptions = getSnowplowOptions(
      appId,
      hasCookieConsent,
      plugins
    );

    this.snowplow = newTracker(trackerId, host, snowplowOptions);

    if (pagePingTracking && pagePingOptions) {
      // Enables page activity tracking (sends page pings to the Collector regularly).
      enableActivityTracking(pagePingOptions);
    }

    /** enable required plugins */

    const { formTracking, linkTracking, errorTracking } = plugins;

    if (formTracking) {
      enableFormTracking();
    }
    if (linkTracking) {
      enableLinkClickTracking();
    }
    if (errorTracking) {
      enableErrorTracking();
    }
  }

  private get trackingEnabled(): boolean {
    const hasGPCConsent = this.options.respectGlobalPrivacyControl
      ? hasGlobalPrivacyControlConsent()
      : true;

    return hasGPCConsent && !doNotTrackEnabled();
  }

  /**
   * Track the user defined custom event
   *
   * @param {eventName} eventName -  Name of the custom event
   * @param {eventAttributes} eventAttributes -  Additional information for custom event
   * @returns {void}
   */
  public track(
    eventName: string,
    { ...eventAttributes }: Record<string, any> = ({} = {})
  ): void {
    if (!this.trackingEnabled) return;
    trackSelfDescribingEvent({
      event: {
        schema: SCHEMAS.CUSTOM_EVENT,
        data: {
          name: eventName,
          props: { ...eventAttributes },
        },
      },
    });
  }

  /**
   * Send error as self-describing event
   *
   * @param errorAttributes  - The event information
   * @returns {void}
   */
  public trackError({
    ...errorAttributes
  }: ErrorEventProperties & CommonEventProperties): void {
    if (!this.trackingEnabled) return;

    // if error tracking plugin is not enabled, do not capture the error
    if (!this.options.plugins?.errorTracking) {
      console.warn('GitLab: Error tracking plugin is not enabled');
      return;
    }
    _trackError({ ...errorAttributes });
  }

  /**
   * Set cookie consent
   */
  addCookieConsent() {
    disableAnonymousTracking({
      stateStorageStrategy: 'cookieAndLocalStorage',
    });
  }

  /**
   * Tracks the page views
   *
   * @param {event} event - The page view event.
   * @returns {void}
   */
  public page(event?: PageViewEvent & CommonEventProperties): void {
    if (!this.trackingEnabled) return;
    trackPageView(event);
  }

  /**
   * Set the business defined userId and other user attributes
   *
   * @param {userId} userId -  The business-defined user ID
   * @param {userAttributes} userAttributes -  The business-defined user attributes
   * @returns {void}
   */
  public identify(
    userId: string,
    userAttributes: Record<string, any> = ({} = {})
  ): void {
    if (!this.trackingEnabled) return;

    setUserId(userId);

    if (!isEmpty(userAttributes)) {
      let userContext = {
        schema: SCHEMAS.USER_CONTEXT,
        data: userAttributes,
      };
      addGlobalContexts([userContext]);
    }
  }
}
